.str { color: #9daa7e; }
.kwd { color: #d5b57c; }
.com { color: #726d73; }
.typ { color: #dd7e5e; }
.lit { color: #fcf0a4; }
.pun, .opn, .clo { color: #a78353; }
.pln { color: #889dbc; }
.tag { color: #d5b57c; }
.atn { color: #dd7e5e; }
.atv { color: #9daa7e; }
.dec { color: #dd7e5e; }